/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#ifdef G_OS_WIN32
extern FILE	*stdout_fp, *stderr_fp;
#endif

#if USE_GUI
# ifdef G_OS_WIN32
void online_help()
{
	char		browser_cmd[512], tmp[1024];
	unsigned int	i, count, start = 0, end = 0;
	char		link[] = APP_SUPPORT_WEBSITE;
	char		*argv[3];
	GPid		pid;
	GError		*error = NULL;

	if (get_default_browser_from_win32registry(browser_cmd) == WIN32REGKEY_FOUND) {
		/* find 1st str in browser_cmd */
		str_n_cpy(tmp, browser_cmd, 511);
		for (i = 0, count = 0; i < strlen(tmp); i++) {
			if (tmp[i] == '"') {
				count++;
				if (count == 1)
					start = i + 1;
				else if (count == 2)
					end = i - 1;
			}
		}
		str_n_cpy(browser_cmd, tmp + start, MIN((end - start + 1), 511));
		/* launch browser */
		fprintf(STD_OUT, "Spawning: %s %s\n", browser_cmd, link);
		argv[0] = browser_cmd;
		argv[1] = link;
		argv[2] = NULL;
		if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &pid, &error))
			warning("News: Can't create process ", argv[0], " - ", error->message);
		else
			g_spawn_close_pid(pid);
	}
	return;
}
# else
void online_help()
{
	char		tmp1[FILENAME_MAXLEN + 1];
	char		*help_url = APP_SUPPORT_WEBSITE;
	char		*argv[3];
	GPid		pid;
	GError		*error = NULL;

	fprintf(STD_OUT, "Spawning: %s %s\n", get_params()->open_link_cmd, help_url);
	str_n_cpy(tmp1, get_params()->open_link_cmd, FILENAME_MAXLEN);
	argv[0] = tmp1;
	argv[1] = help_url;
	argv[2] = NULL;
	if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &pid, &error)) {
		warning("News: Can't create process ", argv[0], " - ", error->message);
		info_win("Have you correctly linked News to your browser ?");
	} else
		g_spawn_close_pid(pid);
	return;
}
# endif
#endif

/*
 * this func's prototype is in libetm / error.h
 * this func is not defined in libetm but in app and it handles errors
 * from libetm and app
 */
int big_error(int error_code, const char *str1, const char *str2, const char *str3, const char *str4)
{
	char	error_str[128 * 5 + 64 + 1];

	get_gr_env()->suspend_rq = TRUE;

	str_n_cpy(error_str, "CRITICAL ERROR: ", 16);
	if (error_code <= LIB_LASTERRORCODE)
		str_n_cat(error_str, "Libetm:", 16);
	else
		str_n_cat(error_str, "News:", 16);
	if (str1[0] != '\0') {
		str_n_cat(error_str, " ", 1);
		str_n_cat(error_str, str1, 127);
	}
	if (str2[0] != '\0') {
		str_n_cat(error_str, " ", 1);
		str_n_cat(error_str, str2, 127);
	}
	if (str3[0] != '\0') {
		str_n_cat(error_str, " ", 1);
		str_n_cat(error_str, str3,127);
	}
	if (str4[0] != '\0') {
		str_n_cat(error_str, " ", 1);
		str_n_cat(error_str, str4, 127);
	}
	if (error_code <= LIB_LASTERRORCODE) {
		str_n_cat(error_str, ": ", 2);
		str_n_cat(error_str, get_lib_error_str(error_code), 126);
	}
	str_n_cat(error_str, " - Will quit now", 32);

	if (STD_ERR != NULL) {
		fprintf(STD_ERR, "%s\n", error_str);
		fflush(NULL);
	}
#if USE_GUI
	info_win(error_str);
#endif
	if (get_resource()->fp != NULL)
		fclose(get_resource()->fp);
	free2(get_gr_env());
	free2(get_resource());
	free2(get_params());
	exit(error_code);
}

void warning(const char *str1, const char *str2, const char *str3, const char *str4)
{
	char	warning_str[128 * 4 + 1];

	warning_str[0] = '\0';
	if (str1[0] != '\0')
		str_n_cat(warning_str, str1, 128);
	if (str2[0] != '\0') {
		str_n_cat(warning_str, " ", 1);
		str_n_cat(warning_str, str2, 127);
	}
	if (str3[0] != '\0') {
		str_n_cat(warning_str, " ", 1);
		str_n_cat(warning_str, str3, 127);
	}
	if (str4[0] != '\0') {
		str_n_cat(warning_str, " ", 1);
		str_n_cat(warning_str, str4, 127);
	}
	fprintf(STD_ERR, "%s\n", warning_str);
#if USE_GUI
	info_win(warning_str);
#endif
	return;
}

void dump_font_list()
{
#define NFAMMAX		1024
#define NFACEMAX	16

	PangoFontFamily		**font_fam;
	PangoFontFamily		*font_fam2[NFAMMAX];
	PangoFontFace		**font_face;
	PangoFontFace		*font_face2[NFACEMAX];
	PangoFontDescription	*font_desc;
	char			*font[NFAMMAX * NFACEMAX + 1], *tmp;
	int			n_fam, n_face, i, j, k, overflow, min;

	fprintf(STD_OUT, "List of available fonts on this system:\n");

	font_fam = font_fam2;
	font_face = font_face2;
	overflow = FALSE;
	k = 0;

	pango_font_map_list_families(	pango_cairo_font_map_get_default(),
					&font_fam, &n_fam);
	if (n_fam >= NFAMMAX)
		overflow = TRUE;
	for (i = 0; i < n_fam && i < NFAMMAX; i++) {
		pango_font_family_list_faces(font_fam[i], &font_face, &n_face);
		if (n_face >= NFACEMAX)
			overflow = TRUE;
		for (j = 0; j < n_face && j < NFACEMAX; j++) {
			font_desc = pango_font_face_describe(font_face[j]);
			font[k++] = pango_font_description_to_string(font_desc);
			pango_font_description_free(font_desc);
		}
	}
	font[k] = NULL;
/*
 * not sure if we must g_free() sth?
 */
	/*
	 * sort array (selection sort)
	 */
	for (i = 0; i < k; i++) {
		min = i;
		for (j = i  + 1; j < k; j++) {
			if (strcmp(font[min], font[j]) > 0)
				min = j;
		}
		tmp = font[i];
		font[i] = font[min];
		font[min] = tmp;
	}

	for (i = 0; i < k; i++) {
		if (font[i] != NULL)
			fprintf(STD_OUT, "%s\n", font[i]);
		else
			break;
	}

	if (overflow)
		fprintf(STD_ERR,
			"In function dump_font_list(): need to allocate more space for array\n"
			"Can't dump full list (more than %d available fonts)\n", k);
	else
		fprintf(STD_OUT, "(%d available fonts)\n", k);

	return;
}

#ifdef G_OS_WIN32
/* browser_cmd must be able to store 511 chars */
int get_default_browser_from_win32registry(char *browser_cmd)
{
HKEY		hkey;
DWORD		type = REG_SZ, buf_size = 512;
LONG		result;

	if ((result = RegOpenKeyEx(HKEY_CLASSES_ROOT, "http\\shell\\open\\command",
			0L, KEY_QUERY_VALUE, &hkey)) == ERROR_SUCCESS) {
		if (RegQueryValueEx(hkey, NULL, NULL, &type, (unsigned char*)browser_cmd,
				&buf_size) == ERROR_SUCCESS) {
			RegCloseKey(hkey);
			return WIN32REGKEY_FOUND;
		} else {
			RegCloseKey(hkey);
			return WIN32REGKEY_NOT_FOUND;
		}
	} else {
		if (result == ERROR_FILE_NOT_FOUND)
			return WIN32REGKEY_NOT_FOUND;
		else
			return WIN32REGKEY_OTHER_ERROR;
	}
}

const char *get_appdata_dir()
{
	static TCHAR	appdata_dir[MAX_PATH];
	static int	i = 0;

	if (i == 0) {
		i++;
		if (SHGetFolderPath(NULL, CSIDL_APPDATA, NULL, 0, appdata_dir) != S_OK)
			i++;
	}
	if (i == 1)
		return (const char *)appdata_dir;
	else
		return NULL;
}

const char *get_progfiles_dir()
{
	static TCHAR	progfiles_dir[MAX_PATH];
	static int	i = 0;

	if (i == 0) {
		i++;
		if (SHGetFolderPath(NULL, CSIDL_PROGRAM_FILES, NULL, 0, progfiles_dir) != S_OK)
			i++;
	}
	if (i == 1)
		return (const char *)progfiles_dir;
	else
		return NULL;
}
#endif

/* this avoid troubles in case of an empty string inside layout */
void pango_layout_get_pixel_size2(PangoLayout *p_layout, int *layout_width, int *layout_height)
{
	if (strlen(pango_layout_get_text(p_layout)) > 0)
		pango_layout_get_pixel_size(p_layout, layout_width, layout_height);
	else {
		*layout_width = 0;
		*layout_height = 0;
		/*big_error(RENDER_OTHER_ERROR,
			"news_misc.c:", "pango_layout_get_pixel_size2():", "Text length = 0", "");*/
		fprintf(STD_ERR,
			"news_misc.c: pango_layout_get_pixel_size2(): Text length = 0 - Troubles ahead ?");
	}
	return;
}
