/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

#ifdef G_OS_WIN32
extern FILE	*stdout_fp, *stderr_fp;
#endif

void set_default_options(Params *prm)
{
	char	tmp_font[FONT_MAXLEN + 1];

	prm->delay = DELAY;

	prm->shift_size = SHIFTSIZE;

	compact_font(tmp_font, FONTNAME, FONTSIZE);
	str_n_cpy(prm->font_n_s, tmp_font, FONT_MAXLEN);
	get_gdk_color_from_hexstr(FGCOLOR, &prm->fg_color, &prm->fg_color_alpha);
	get_gdk_color_from_hexstr(BGCOLOR, &prm->bg_color, &prm->bg_color_alpha);

	prm->win_x = WIN_X;
	prm->win_y = WIN_Y;
	prm->win_w = WIN_W;
	prm->win_h = WIN_H;

	prm->windec = WINDEC;
	prm->always_on_top = ALWAYSONTOP;
	prm->win_transparency = WINTRANSPARENCY;

	prm->shadow = SHADOW;
	prm->shadow_offset_x = SHADOWOFFSET_X;
	prm->shadow_offset_y = SHADOWOFFSET_Y;
	prm->shadow_fx = SHADOWFX;

	str_n_cpy(prm->line_delimiter, LINEDELIMITER, DELIMITER_MAXLEN);
	str_n_cpy(prm->cutline_delimiter, CUTLINEDELIMITER, DELIMITER_MAXLEN);

	prm->rss_refresh = RSSREFRESH;
	prm->rss_title = RSSTITLE;
	str_n_cpy(prm->rss_title_delimiter, RSSTITLEDELIMITER, DELIMITER_MAXLEN);
	prm->rss_description = RSSDESCRIPTION;
	str_n_cpy(prm->rss_description_delimiter, RSSDESCRIPTIONDELIMITER, DELIMITER_MAXLEN);

	str_n_cpy(prm->homefeed, HOMEFEED, FILENAME_MAXLEN);

	str_n_cpy(prm->open_link_cmd, OPENLINKCMD, FILENAME_MAXLEN);
	str_n_cpy(prm->open_link_args, OPENLINKARGS, FILENAME_MAXLEN);

	prm->clock = CLOCK;
	get_gdk_color_from_hexstr(CFGCOLOR, &prm->clock_fg_color,
						&prm->clock_fg_color_alpha);
	get_gdk_color_from_hexstr(CBGCOLOR, &prm->clock_bg_color,
						&prm->clock_bg_color_alpha);
	compact_font(tmp_font, CFONTNAME, CFONTSIZE);
	str_n_cpy(prm->clock_font_n_s, tmp_font, FONT_MAXLEN);

	prm->special_chars = SPECIALCHARS;
	prm->new_page_char = NEWPG;
	prm->tab_char = TABCH;
	return;
}

/*
 * load conf file if any & set up - returns 0 if ok
 * store options in a string array and initialize a ptr arrays that matches strings
 * we assume we have up to NOPTIONSMAX options, OPTION_MAXLEN char long max each
 */
int get_config_file_options(Params *prm)
{
	FILE		*conf_fp;
#ifdef G_OS_WIN32
	gchar		*win32_filename;
#endif
	char		options_array[NOPTIONSMAX][OPTION_MAXLEN + 2], tmp[TMPSTR_SIZE + 1];
	char		*ptr[NOPTIONSMAX + 1];
	int		i, j = 0;

#ifndef G_OS_WIN32
	if ((conf_fp = g_fopen(config_file_fullname(), "rb")) != NULL) {
#else
	win32_filename = g_win32_locale_filename_from_utf8(config_file_fullname());
	if ((conf_fp = g_fopen(win32_filename, "rb")) != NULL) {
#endif
		/*
		 * parse conf file
		 */
		for (i = 0; ; i++) {
			/*
			 * here fgets() read OPTION_MAXLEN + 1 char max including
			 * '\n' then add '\0' (if I'm right...)
			 */
			if (i >= NOPTIONSMAX) {
				ptr[i] = NULL;
				snprintf(tmp, TMPSTR_SIZE + 1,
					"Too many lines in configuration file: %s "
					"(max = %d)\nWon't parse further than limit\n",
					config_file_fullname(), NOPTIONSMAX);
				warning(tmp, "", "", "");
				break;
			}

			if (fgets(options_array[i], OPTION_MAXLEN + 2, conf_fp) == NULL) {
				ptr[i] = NULL;
				break;
			} else {
				j = strlen(options_array[i]);
				options_array[i][j - 1] = '\0';	/* we remove trailing '\n' */
				ptr[i] = options_array[i];	/* so now options max length = OPTION_MAXLEN */
			}
		}

		if (i > 0)
			j = parse_options_array(prm, i - 1, (const char **)ptr);
		else
			j = 0;

		fclose(conf_fp);
	} else
#ifndef G_OS_WIN32
		fprintf(STD_ERR, "Can't read configuration file: %s - %s\n",
			config_file_fullname(), strerror(errno));
#else
		fprintf(STD_ERR, "Can't read configuration file: %s - %s\n",
			win32_filename, strerror(errno));
	g_free(win32_filename);
#endif
	return j;
}

/*
 * options array parser (NULL terminated ptrs array)
 * we assume we have up to NOPTIONSMAX options, OPTION_MAXLEN char long max each
 */
int parse_options_array(Params *prm, int n_options, const char *option[])
{
	int	i, exit_status = OPTION_OK;

	if (n_options > NOPTIONSMAX) {
		n_options = NOPTIONSMAX;
		fprintf(STD_ERR,
			"Too many options (max = %d) - Won't parse further than limit\n",
			NOPTIONSMAX);
			exit_status = OPTION_TOO_MANY;
	}

	for (i = 0; i < n_options && option[i] != NULL; i++)
		if (get_name_value_from_option(prm, option[i]) != OPTION_OK)
			exit_status = OPTION_ERROR;

	return exit_status;
}

int get_name_value_from_option(Params *prm, const char *option)
{
	/*
	 * options are splitted into name and value
	 * font -> fname is FONT_NAME_MAXLEN char max long, fsize FONT_SIZE_MAXLEN,
	 * plus 1 extra space so font_n_s (font name size) is up to
	 * FONT_MAXLEN = FONT_NAME_MAXLEN + 1 + FONT_SIZE_MAXLEN char long
	 * OPTION_MAXLEN = OPTION_NAME_MAXLEN + OPTION_VALUE_MAXLEN + 4 (-name="value")
	 *
	 * === if unsure, check this in news.h ===
	 */
	char	name[OPTION_NAME_MAXLEN + 1], value[OPTION_VALUE_MAXLEN + 1];
	char	fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
	char	cfname[FONT_NAME_MAXLEN + 1], cfsize[FONT_SIZE_MAXLEN + 1];
	char	tmp[OPTION_MAXLEN + 1];
	char	tmp_font[FONT_MAXLEN + 1];
	int	i, exit_status = OPTION_OK;

	split_font(prm->font_n_s, fname, fsize);
	split_font(prm->clock_font_n_s, cfname, cfsize);
	/*
	 * split option into name & value if any
	 */
	name[0] = '\0';
	value[0] = '\0';
	if (option[0] == '-') {
		str_n_cpy(tmp, option + 1, OPTION_MAXLEN);
		for (i = 0; i < OPTION_MAXLEN - 1; i++)
			if (tmp[i] == '=') {
				tmp[i] = '\0';
				str_n_cpy(name, tmp, OPTION_NAME_MAXLEN);
				str_n_cpy(value, tmp + i + 1, OPTION_VALUE_MAXLEN);
				break;
			}
	} else if (option[0] != '\0') {
		fprintf(STD_ERR, "Invalid option: %s\n", option);
		return (exit_status = OPTION_INVALID);
	}

	/*
	 * check option name and get value if any
	 */
	if (strcmp(name, "delay") == 0)
		prm->delay = atoi(value);
	else if (strcmp(name, "shiftsize") == 0)
		prm->shift_size = atoi(value);
	else if (strcmp(name, "fontname") == 0)
		str_n_cpy(fname, value, FONT_NAME_MAXLEN);
	else if (strcmp(name, "fontsize") == 0) {
		str_n_cpy(fsize, value, FONT_SIZE_MAXLEN);
		if (atoi(fsize) > FONT_MAXSIZE) {
			snprintf(fsize, FONT_SIZE_MAXLEN + 1, "%3d", FONT_MAXSIZE);
			fprintf(STD_ERR, "Font size set to %d (can't be above)\n",
				FONT_MAXSIZE);
		}
	} else if (strcmp(name, "fgcolor") == 0) {
		if (!get_gdk_color_from_hexstr(value, &prm->fg_color,
						&prm->fg_color_alpha)) {
			get_gdk_color_from_hexstr(FGCOLOR, &prm->fg_color,
						&prm->fg_color_alpha);
			exit_status = OPTION_VALUE_INVALID;
		}
	} else if (strcmp(name, "bgcolor") == 0) {
		if (!get_gdk_color_from_hexstr(value, &prm->bg_color,
						&prm->bg_color_alpha)) {
			get_gdk_color_from_hexstr(BGCOLOR, &prm->bg_color,
						&prm->bg_color_alpha);
			exit_status = OPTION_VALUE_INVALID;
		}
	} else if (strcmp(name, "win_x") == 0)
		prm->win_x = atoi(value);
	else if (strcmp(name, "win_y") == 0)
		prm->win_y = atoi(value);
	else if (strcmp(name, "win_w") == 0)
		prm->win_w = atoi(value);
	/*
	 * if win_h is set & != 0, it will override requested font size
	 * with computed one
	 */
	else if (strcmp(name, "win_h") == 0) {
		prm->win_h = atoi(value);
		if (prm->win_h > 0 && prm->win_h < DRWA_HEIGHT_MIN)
			prm->win_h = DRWA_HEIGHT_MIN;
	} else if (strcmp(name, "windec") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->windec = value[0];
	} else if (strcmp(name, "alwaysontop") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->always_on_top = value[0];
	} else if (strcmp(name, "wintransparency") == 0) {
		prm->win_transparency = (gdouble)atoi(value) / 10;
		if (prm->win_transparency < 0.0 || prm->win_transparency > 1.0) {
			prm->win_transparency = WINTRANSPARENCY;
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
			
		}
	} else if (strcmp(name, "shadow") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->shadow = value[0];
	}
	/* should check value here */
	else if (strcmp(name, "shadowoffset_x") == 0)
		prm->shadow_offset_x = atoi(value);
	/* should check value here */
	else if (strcmp(name, "shadowoffset_y") == 0)
		prm->shadow_offset_y = atoi(value);
	else if (strcmp(name, "shadowfx") == 0)
		prm->shadow_fx = atoi(value);
	else if (strcmp(name, "linedelimiter") == 0)
		str_n_cpy(prm->line_delimiter, value, DELIMITER_MAXLEN);
	else if (strcmp(name, "cutlinedelimiter") == 0)
		str_n_cpy(prm->cutline_delimiter, value, DELIMITER_MAXLEN);
	else if (strcmp(name, "rssrefresh") == 0)
		prm->rss_refresh = atoi(value);
	else if (strcmp(name, "rsstitle") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->rss_title = value[0];
	} else if (strcmp(name, "rsstitledelimiter") == 0)
		str_n_cpy(prm->rss_title_delimiter, value, DELIMITER_MAXLEN);
	else if (strcmp(name, "rssdescription") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->rss_description = value[0];
	} else if (strcmp(name, "rssdescriptiondelimiter") == 0)
		str_n_cpy(prm->rss_description_delimiter, value, DELIMITER_MAXLEN);
	else if (strcmp(name, "homefeed") == 0)
		str_n_cpy(prm->homefeed, value, FILENAME_MAXLEN);
	else if (strcmp(name, "openlinkcmd") == 0)
		str_n_cpy(prm->open_link_cmd, value, FILENAME_MAXLEN);
	else if (strcmp(name, "openlinkargs") == 0)
		str_n_cpy(prm->open_link_args, value, FILENAME_MAXLEN);
	else if (strcmp(name, "clock") == 0) {
		if (strcmp(value, "l") != 0 && strcmp(value, "r") != 0 \
					&& strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->clock = value[0];
	} else if (strcmp(name, "clockfontname") == 0)
		str_n_cpy(cfname, value, FONT_NAME_MAXLEN);
	else if (strcmp(name, "clockfontsize") == 0)
		str_n_cpy(cfsize, value, FONT_SIZE_MAXLEN);
	else if (strcmp(name, "clockfgcolor") == 0) {
		if (!get_gdk_color_from_hexstr(value, &prm->clock_fg_color,
					&prm->clock_fg_color_alpha)) {
			get_gdk_color_from_hexstr(CFGCOLOR, &prm->clock_fg_color,
					&prm->clock_fg_color_alpha);
			exit_status = OPTION_VALUE_INVALID;
		}
	} else if (strcmp(name, "clockbgcolor") == 0) {
		if (!get_gdk_color_from_hexstr(value, &prm->clock_bg_color,
					&prm->clock_bg_color_alpha)) {
			get_gdk_color_from_hexstr(CBGCOLOR, &prm->clock_bg_color,
					&prm->clock_bg_color_alpha);
			exit_status = OPTION_VALUE_INVALID;
		}
	} else if (strcmp(name, "specialchars") == 0) {
		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
			fprintf(STD_ERR, "Invalid value: %s\n", option);
			exit_status = OPTION_VALUE_INVALID;
		} else
			prm->special_chars = value[0];
	} else if (strcmp(name, "newpgchar") == 0)
		prm->new_page_char = value[0];
	else if (strcmp(name, "tabchar") == 0)
		prm->tab_char = value[0];
	/* this option is only set from command line */
	else if (strcmp(name, "instance-id") == 0)
		;/* does nothing but avoid an unknown option warning/error */
	else {
		fprintf(STD_ERR, "Unknown option: %s\n", option);
		exit_status = OPTION_UNKNOWN;
	}
	compact_font(tmp_font, fname, fsize);
	str_n_cpy(prm->font_n_s, tmp_font, FONT_MAXLEN);
	compact_font(tmp_font, cfname, cfsize);
	str_n_cpy(prm->clock_font_n_s, tmp_font, FONT_MAXLEN);

	return exit_status;
}

void save_to_config_file(const Params *prm)
{
	FILE		*conf_fp;
#ifdef G_OS_WIN32
	gchar		*win32_filename;
#endif
	/*char		str1[9 + 1], str2[9 + 1], str3[9 + 1], str4[9 + 1];*/
	char		fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
	char		cfname[FONT_NAME_MAXLEN + 1], cfsize[FONT_SIZE_MAXLEN + 1];
	char		tmp[TMPSTR_SIZE + 1];

#ifndef G_OS_WIN32
	if ((conf_fp = g_fopen(config_file_fullname(), "wb")) != NULL) {
#else
	win32_filename = g_win32_locale_filename_from_utf8(config_file_fullname());
	if ((conf_fp = g_fopen(win32_filename, "wb")) != NULL) {
#endif
		split_font(prm->font_n_s, fname, fsize);
		split_font(prm->clock_font_n_s, cfname, cfsize);
		snprintf(tmp, TMPSTR_SIZE + 1, "%s%s",
			"-delay=%d\n-shiftsize=%d\n"
			"-fontname=%s\n-fontsize=%d\n"
			"-fgcolor=%s\n-bgcolor=%s\n"
			"-win_x=%d\n-win_y=%d\n-win_w=%d\n-win_h=%d\n"
			"-windec=%c\n-alwaysontop=%c\n-wintransparency=%d\n"
			"-shadow=%c\n-shadowoffset_x=%d\n-shadowoffset_y=%d\n"
			"-shadowfx=%d\n"
			"-linedelimiter=%s\n-cutlinedelimiter=%s\n",
			"-rssrefresh=%d\n"
			"-rsstitle=%c\n-rsstitledelimiter=%s\n"
			"-rssdescription=%c\n-rssdescriptiondelimiter=%s\n"
			"-homefeed=%s\n"
			"-openlinkcmd=%s\n-openlinkargs=%s\n"
			"-clock=%c\n"
			"-clockfontname=%s\n-clockfontsize=%d\n"
			"-clockfgcolor=%s\n-clockbgcolor=%s\n"
			"-specialchars=%c\n-newpgchar=%c\n-tabchar=%c\n\n");
		fprintf(conf_fp, tmp,
			prm->delay, prm->shift_size, fname, atoi(fsize),
			get_hexstr_from_gdk_color(&prm->fg_color, &prm->fg_color_alpha),
			get_hexstr_from_gdk_color(&prm->bg_color, &prm->bg_color_alpha),
			prm->win_x, prm->win_y, prm->win_w, prm->win_h,
			prm->windec, prm->always_on_top, (int)(prm->win_transparency * 10),
			prm->shadow, prm->shadow_offset_x, prm->shadow_offset_y, prm->shadow_fx,
			prm->line_delimiter, prm->cutline_delimiter,
			prm->rss_refresh, prm->rss_title, prm->rss_title_delimiter,
			prm->rss_description, prm->rss_description_delimiter,
			prm->homefeed, prm->open_link_cmd, prm->open_link_args,
			prm->clock, cfname, atoi(cfsize),
			get_hexstr_from_gdk_color(&prm->clock_fg_color, &prm->clock_fg_color_alpha),
			get_hexstr_from_gdk_color(&prm->clock_bg_color, &prm->clock_bg_color_alpha),
			prm->special_chars, prm->new_page_char, prm->tab_char);
		fclose(conf_fp);
	} else
#ifndef G_OS_WIN32
		warning("Can't save configuration file:", config_file_fullname(),
			"-", strerror(errno));
#else
		warning("Can't save configuration file:", win32_filename,
			"-", strerror(errno));
	g_free(win32_filename);
#endif

	return;
}

/*
 * options colors are coded as 4 hexa values -> #rrggbbaa
 */
int get_gdk_color_from_hexstr(const char *str, GdkColor *color, guint16 *color_alpha)
{
	char tmp[3], *tailptr;
	int errflag = 0;

	if (strlen(str) != 9 && str[0] != '#')
		errflag = 1;
	else {
		str_n_cpy(tmp, str + 1, 2);
		color->red = strtoul(tmp, &tailptr, 16) * 0x100;
		if (tailptr == tmp)
		errflag = 1;

		str_n_cpy(tmp, str + 3, 2);
		color->green = strtoul(tmp, &tailptr, 16) * 0x100;
		if (tailptr == tmp)
			errflag = 1;

		str_n_cpy(tmp, str + 5, 2);
		color->blue = strtoul(tmp, &tailptr, 16) * 0x100;
		if (tailptr == tmp)
			errflag = 1;

		str_n_cpy(tmp, str + 7, 2);
		*color_alpha = strtoul(tmp, &tailptr, 16) * 0x100;
		if (tailptr == tmp)
			errflag = 1;
	}

	if (errflag != 0) {
		fprintf(STD_ERR, "Invalid color: %s\n", str);
		return FALSE;
	} else
		return TRUE;
}

/*
 * options colors are coded as 4 hexa values -> #rrggbbaa
 * *** allow up to 64 simultaneous calls ***
 */
const char *get_hexstr_from_gdk_color(const GdkColor *color, const guint16 *color_alpha)
{
	static char	str[64][10];
	static int	count = -1;

	count++;
	count &= 63;
	snprintf(str[count], 10, "#%02x%02x%02x%02x", color->red / 256, color->green / 256,
					color->blue / 256, *color_alpha / 256);
	if (strlen(str[count]) != 9)
		return NULL;
	else
		return (const  char *)str[count];
}

/*
 * space needed for font_name: FONT_NAME_MAXLEN + 1 bytes / font_size: FONT_SIZE_MAXLEN + 1 bytes
 */
void split_font(const char *font_name_size, char *font_name, char *font_size)
{
	char		tmp[FONT_MAXLEN + 1];
	int		i;

	if (font_name_size[0] == '\0') {
		font_name[0] = '\0';
		font_size[0] = '\0';
	} else {
		str_n_cpy(tmp, font_name_size, FONT_MAXLEN);

		for (i = strlen(tmp) - 1; i > 0 ; i--)
			if (tmp[i] == ' ') {
				tmp[i] = '\0';
				break;
			}

		str_n_cpy(font_name, tmp, FONT_NAME_MAXLEN);
		str_n_cpy(font_size, tmp + strlen(font_name) + 1, FONT_SIZE_MAXLEN);
	}
	return;
}

/*
 * space needed for font_name_size: FONT_NAME_MAXLEN + 1 space + FONTSIZENMAXLEN + 1 bytes
 */
void compact_font(char *font_name_size, const char *font_name, const char *font_size)
{
	if (font_name[0] == '\0' && font_size[0] == '\0') {
		font_name_size[0] = '\0';
	} else {
		str_n_cpy(font_name_size, font_name, FONT_NAME_MAXLEN);
		str_n_cat(font_name_size, " ", 1);
		str_n_cat(font_name_size, font_size, FONT_SIZE_MAXLEN);
	}
	return;
}

const char *config_file_fullname()
{
	static char	str[512];

#ifndef G_OS_WIN32
	str_n_cpy(str, usr_home_dir(), 511 - 64);
	str_n_cat(str, "/"CONFIGFILE, 64 - 2);
#else
	str_n_cpy(str, get_appdata_dir(), 511 - 64);
	str_n_cat(str, "\\"CONFIGFILE, 64 - 2);
#endif
	if (get_instance_id() != 0)
		str_n_cat(str, itoa2(get_instance_id()), 2);
	return (const char*)str;
}

const char *usr_home_dir()
{
	static char	str[512] = "";

#ifndef G_OS_WIN32
	return (const char *)str_n_cpy(str, getpwuid(getuid())->pw_dir, 511);
#endif
	/* for win32 version, no home dir */
	return (const char *)str;
}
