/*
 *	NEWS-0.4.4 - RSS Feed Ticker - Copyright (C) Emmanuel Thomas-Maurin 2009-2010
 *	< manutm007@gmail.com >
 *
 * 	This program is free software: you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation, either version 3 of the License, or
 * 	(at your option) any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "news.h"

/*
 * [feed url + (with '>') title] max length = FILENAME_MAXLEN because
 * we include feed title in the list
 */
#define URL_ENTRY_LENGTH	100

#if USE_GUI
static GtkWidget	*dialog, *entry_url;
static GrEnv		*env;
static gchar		home_feed[FILENAME_MAXLEN + 1];
static int		rank;

static gint esc_key_pressed(GtkWidget *dialog2, GdkEventKey *event, gpointer unused)
{
	unused = unused;
	if (event->keyval == GDK_Escape) {
		gtk_dialog_response(GTK_DIALOG(dialog2), GTK_RESPONSE_CANCEL);
		return TRUE;
	} else
		return FALSE;
}

static void force_quit_dialog()
{
	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_CANCEL);
}

/* tree selection callback - copy selected url to entry_url (override) only if entry_url is not empty */
static gint tree_selection_changed(GtkTreeSelection *selection, GdkEventAny *event, gpointer unused)
{
	GtkTreeModel		*tree_model = NULL;
	GtkTreeIter		iter;
	gchar			*str_title, *str_url;

	enum {COLUMN_INT, COLUMN_STRING_TITLE, COLUMN_STRING_URL, N_COLUMNS};

	event = event;
	unused = unused;
	/*if (gtk_entry_get_text(GTK_ENTRY(entry_url))[0] != '\0') {*/
		env->suspend_rq = TRUE;
		if (gtk_tree_selection_get_selected(selection, &tree_model, &iter)) {
			gtk_tree_model_get(tree_model, &iter,
				COLUMN_INT, &rank,
				COLUMN_STRING_TITLE, &str_title,
				COLUMN_STRING_URL, &str_url,
				-1);
			gtk_entry_set_text(GTK_ENTRY(entry_url), str_url);
			g_free(str_title);
			g_free(str_url);
		}
		env->suspend_rq = FALSE;
		gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_NONE);
		return TRUE;
	/*} else
		return FALSE;*/
}

/* to catch double-click on tree view */
static gint double_click_on_tree_view(GtkTreeView *tree_view, GtkTreePath *tree_path, gpointer unused)
{
	GtkTreeModel		*tree_model = NULL;
	GtkTreeIter		iter;
	gchar			*str_title, *str_url;

	enum {COLUMN_INT, COLUMN_STRING_TITLE, COLUMN_STRING_URL, N_COLUMNS};

	unused = unused;
	env->suspend_rq = TRUE;
	tree_model = gtk_tree_view_get_model(tree_view);
	if (gtk_tree_model_get_iter(tree_model, &iter, tree_path)) {
		gtk_tree_model_get(tree_model, &iter,
			COLUMN_INT, &rank,
			COLUMN_STRING_TITLE, &str_title,
			COLUMN_STRING_URL, &str_url,
			-1);
		gtk_entry_set_text(GTK_ENTRY(entry_url), str_url);
		g_free(str_title);
		g_free(str_url);
	}
	env->suspend_rq = FALSE;
	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
	return TRUE;
}

static gint enter_key_pressed_in_entry(GtkWidget *widget, GdkEventKey *event, gpointer unused)
{
	widget = widget;
	event = event;
	unused = unused;
	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
	return TRUE;
}

/* need to filter tab_key_pressed events */
/*static gint tab_key_pressed_in_treeview(GtkWidget *tree_view, GdkEventKey *event, gpointer unused)
{
	tree_view = tree_view;
	unused = unused;
	if (event->keyval == GDK_Tab)
		return TRUE;
	else
		return FALSE;
}*/

/*
 * open a dialog with a list of urls to choose from and an entry_url
 * (with feed titles)
 */
void get_new_url(Resource *resrc, const Params *prm, int *rss_ttl)
{
	GtkWidget		*vbox, *hbox, *label[2], *sc_win, *tree_view = NULL;
	GtkWidget		*cancel_but, *remove_but, *home_but, *ok_but;
	GtkListStore		*list_store = NULL;
	GtkTreeModel		*tree_model = NULL;
	GtkTreeIter		iter;
	GtkCellRenderer		*renderer1 = NULL, *renderer2 = NULL;
	GtkTreeViewColumn	*column1 = NULL, *column2 = NULL;
	GtkTreeSelection	*selection = NULL;
	GtkRequisition		req;
	gint			list_height;
	gint			response;
	Resource		resrc_bak;
	char			url_array[NURLMAX + 1][FILENAME_MAXLEN + 1];
	char			*p_url[NURLMAX + 1];
	int			rss_status;
	char			tmp[TMPSTR_SIZE + 1], tmp2[256];
	char			rss_feed_url[FILENAME_MAXLEN + 1];
	char			rss_feed_title[FEED_TITLE_MAXLEN + 1];
	char			rss_feed_link[FILENAME_MAXLEN + 1];
	char			rss_feed_ttl[32];
	int			i, j;

	enum {COLUMN_INT, COLUMN_STRING_TITLE, COLUMN_STRING_URL, N_COLUMNS};

	env = get_gr_env();

	gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);

	dialog = gtk_dialog_new_with_buttons("RSS Feed Picker",
		GTK_WINDOW(env->win), GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT, NULL);

	cancel_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
	remove_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_NO);
	home_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_HOME, GTK_RESPONSE_YES);
	ok_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);

#ifndef G_OS_WIN32
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), NEWS_ICON, NULL);
#else
	str_n_cpy(tmp, get_progfiles_dir(), TMPSTR_SIZE - 64);
	str_n_cat(tmp, "\\"NEWS_ICON, 64);
	gtk_window_set_icon_from_file(GTK_WINDOW(dialog), tmp, NULL);
#endif
	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(GTK_BOX(vbox)), 15);

	str_n_cpy(home_feed, get_params()->homefeed, FILENAME_MAXLEN);

	/* init p_url[] which will be used by load, sort and save url list functs */
	for (i = 0; i < NURLMAX - 1; i++)
		p_url[i] = url_array[i];
	p_url[NURLMAX] = NULL;

	list_store = gtk_list_store_new(N_COLUMNS, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING);
	if (load_url_list(p_url) == LOAD_URL_LIST_OK) {
		for (i = 0; i < NURLMAX && p_url[i] != NULL; i++) {
			gtk_list_store_append(list_store, &iter);
			/*
			 * store feed url and title (if any)
			 */
			str_n_cpy(rss_feed_url, p_url[i], FILENAME_MAXLEN);
			for (j = 0; j < FILENAME_MAXLEN; j++) {
				if (rss_feed_url[j] == '\0' || rss_feed_url[j] == '\n') {
					rss_feed_title[0] = '\0';
					break;
				} else if (rss_feed_url[j] == TITLE_TAG_CHAR) {
					str_n_cpy(rss_feed_title, rss_feed_url + j + 1,
						FEED_TITLE_MAXLEN);
					rss_feed_url[j] = '\0';
					break;
				}
			}
			gtk_list_store_set(list_store, &iter, COLUMN_INT, i,
				COLUMN_STRING_TITLE, rss_feed_title,
				COLUMN_STRING_URL, rss_feed_url,
				-1);
		}

		tree_view = gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store));
		renderer1 = gtk_cell_renderer_text_new();
		column1 = gtk_tree_view_column_new_with_attributes("RSS Feed Title",
			renderer1, "text", COLUMN_STRING_TITLE, NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column1);
		renderer2 = gtk_cell_renderer_text_new();
		column2 = gtk_tree_view_column_new_with_attributes("RSS Feed URL",
			renderer2, "text", COLUMN_STRING_URL, NULL);
		gtk_tree_view_append_column(GTK_TREE_VIEW(tree_view), column2);

		sc_win = gtk_scrolled_window_new(NULL, NULL);
		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sc_win),
			GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
		gtk_container_add(GTK_CONTAINER(vbox), sc_win);
		gtk_container_add(GTK_CONTAINER(sc_win), tree_view);
		label[0] = gtk_label_new("");
		gtk_container_add(GTK_CONTAINER(vbox), label[0]);
		selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view));
		gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
		gtk_widget_size_request(tree_view, &req);
		list_height = (req.height > 400) ? 400 : req.height;
		gtk_widget_set_size_request(sc_win, -1, list_height);

		g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(tree_selection_changed), NULL);
		g_signal_connect(G_OBJECT(tree_view), "row_activated", G_CALLBACK(double_click_on_tree_view), NULL);
		/*g_signal_connect(G_OBJECT(tree_view), "key-press-event", G_CALLBACK(tab_key_pressed_in_treeview), NULL);*/
	} else
		p_url[0] = NULL;

	hbox = gtk_hbox_new(FALSE, 0);
	label[1] = gtk_label_new("Enter new URL: ");
	gtk_box_pack_start(GTK_BOX(hbox), label[1], FALSE, FALSE, 0);
	entry_url = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(entry_url), FILENAME_MAXLEN);
	gtk_entry_set_width_chars(GTK_ENTRY(entry_url), URL_ENTRY_LENGTH);
	gtk_box_pack_start(GTK_BOX(hbox), entry_url, FALSE, FALSE, 0);
	gtk_container_add(GTK_CONTAINER(vbox), hbox);

	g_signal_connect(G_OBJECT(entry_url), "activate", G_CALLBACK(enter_key_pressed_in_entry), NULL);
	g_signal_connect(G_OBJECT(dialog), "key-press-event", G_CALLBACK(esc_key_pressed), NULL);
	g_signal_connect(G_OBJECT(dialog), "delete_event", G_CALLBACK(force_quit_dialog), NULL);

	gtk_widget_show_all(dialog);
	gtk_widget_grab_focus(tree_view);

	/* backup last valid opened resource (if any) */
	str_n_cpy(resrc_bak.id, resrc->id, FILENAME_MAXLEN);
	gtk_entry_set_text(GTK_ENTRY(entry_url), "");
	rank = -1;

	while ((response = gtk_dialog_run(GTK_DIALOG(dialog))) != GTK_RESPONSE_CANCEL) {
		/* GTK_RESPONSE_NO is for 'remove selected url' */
		if (response == GTK_RESPONSE_NO) {
			if (rank == -1)
				warning("You must select an URL first", "", "", "");
			else if (gtk_tree_selection_get_selected(selection, &tree_model, &iter)) {
				if (question_win("Remove selected URL from list ?") == YES) {
					gtk_list_store_remove(list_store, &iter);
					remove_url_from_list(p_url, rank);
					save_url_list(p_url);
					rank = -1;
				}
			}
		} else if (response == GTK_RESPONSE_YES)	/* select home feed */
			gtk_entry_set_text(GTK_ENTRY(entry_url), (const gchar *)home_feed);
		else if (response == GTK_RESPONSE_OK) {
			if (gtk_entry_get_text(GTK_ENTRY(entry_url))[0] == '\0' && rank == -1) {
				warning("You must enter or select an URL first", "", "", "");
				continue;
			} else {
				env->suspend_rq = TRUE;
				str_n_cpy(resrc->id, (char *)gtk_entry_get_text(GTK_ENTRY(entry_url)),
					FILENAME_MAXLEN);
			}
			if ((rss_status = get_rss_feed(resrc, prm, rss_ttl)) == RSS_OK) {
				for (i = 0; i < NURLMAX  && p_url[i] != NULL; i++);
				if (i < NURLMAX) {
					p_url[i] = url_array[i];
					str_n_cpy(p_url[i], resrc->id, FILENAME_MAXLEN - FEED_TITLE_MAXLEN - 1);
					snprintf(tmp2, 2, "%c", TITLE_TAG_CHAR);
					str_n_cat(p_url[i], tmp2, 1);
					get_rss_feed_info(resrc->id,
						rss_feed_title, rss_feed_link, rss_feed_ttl);
					str_n_cat(p_url[i], rss_feed_title, FEED_TITLE_MAXLEN);
					p_url[i + 1] = NULL;
					p_url[NURLMAX] = NULL;
					sort_url_list(p_url);
					save_url_list(p_url);
				} else {
					snprintf(tmp, TMPSTR_SIZE + 1,
						"You can't save more than %d URLs\n",
						NURLMAX);
					warning(tmp, "", "", "");
				}
				env->reload_rq = TRUE;
				break;
			} else {
				if (rss_status == RSS_UNPARSABLE)
					warning("Invalid URL:", resrc->id, "", "");
				else if (rss_status == RSS_EMPTY)
					warning("RSS feed is empty:", resrc->id, "", "");
				else {
					snprintf(tmp, TMPSTR_SIZE + 1, "get_rss_feed() error %d",
						rss_status);
					warning(tmp, "", "", "");
				}
				/* restore last valid opened resource (if any) */
				str_n_cpy(resrc->id, resrc_bak.id, FILENAME_MAXLEN);
				env->reload_rq = TRUE;
				break;
			}
		}
	}
	if (list_store != NULL)
		g_object_unref(list_store);
	gtk_widget_destroy(dialog);
	check_main_win_always_on_top();
	env->suspend_rq = FALSE;
	return;
}
#endif
